@@ -81,6 +81,12 @@ module Senna
 
   DEFAULT_PORT                  =      10041
 
+  CTX_MORE                      =      0x0001
+  CTX_TAIL                      =      0x0002
+  CTX_HEAD                      =      0x0004
+  CTX_QUIET                     =      0x0008
+  CTX_QUIT                      =      0x0010
+
   class Index
     attr_reader :handle
     attr_reader :key_size, :flags, :initial_n_segments, :encoding
@@ -817,13 +823,18 @@ module Senna
       return Senna::sen_ctx_recv(@handle)
     end
 
-    def exec(message)
-      send(message)
-      res = []
-      loop {
-        r = recv
-        res << r[1]
-        break if r[2] == 0
+    def exec(*messages)
+      res = nil
+      messages.each {|message|
+        rc = send(message)
+        raise IOError, 'send error' if rc != RC_SUCCESS
+        res = []
+        loop {
+          rc, value, flags = recv
+          raise IOError, 'recv error' if rc != RC_SUCCESS
+          res << value
+          break if (flags & CTX_MORE) == 0
+        }
       }
       res
     end
