@@ -20,12 +20,15 @@
 #include "lex.h"
 #include "ql.h"
 #include "sym.h"
+#include "snip.h"
 #include <stdarg.h>
 #include <stdio.h>
 #include <time.h>
 
 sen_ctx sen_gctx;
 
+/* fixme by 2038 */
+
 sen_rc
 sen_timeval_now(sen_timeval *tv)
 {
@@ -34,16 +37,22 @@ sen_timeval_now(sen_timeval *tv)
   struct _timeb tb;
   time(&t);
   _ftime(&tb);
-  tv->tv_sec = t;
+  tv->tv_sec = (int32_t) t;
   tv->tv_usec = tb.millitm * 1000;
   return sen_success;
 #else /* WIN32 */
-  return gettimeofday(tv, NULL) ? sen_external_error : sen_success;
+  struct timeval t;
+  sen_rc rc = gettimeofday(&t, NULL) ? sen_external_error : sen_success;
+  if (!rc) {
+    tv->tv_sec = (int32_t) t.tv_sec;
+    tv->tv_usec = t.tv_usec;
+  }
+  return rc;
 #endif /* WIN32 */
 }
 
 sen_rc
-sen_timeval_tostr(sen_timeval *tv, char *buf)
+sen_timeval2str(sen_timeval *tv, char *buf)
 {
   struct tm *ltm;
 #ifdef WIN32
@@ -51,7 +60,8 @@ sen_timeval_tostr(sen_timeval *tv, char *buf)
   ltm = localtime(&tvsec);
 #else /* WIN32 */
   struct tm tm;
-  ltm = localtime_r(&tv->tv_sec, &tm);
+  time_t t = tv->tv_sec;
+  ltm = localtime_r(&t, &tm);
 #endif /* WIN32 */
   if (!ltm) { return sen_external_error; }
   snprintf(buf, SEN_TIMEVAL_STR_SIZE - 1, SEN_TIMEVAL_STR_FORMAT,
@@ -61,6 +71,35 @@ sen_timeval_tostr(sen_timeval *tv, char *buf)
   return sen_success;
 }
 
+sen_rc
+sen_str2timeval(const char *str, uint32_t str_len, const char *format, sen_timeval *tv)
+{
+  struct tm tm;
+  const char *r1, *r2;
+  char buf[SEN_TIMEVAL_STR_SIZE];
+  uint32_t uv, len = str_len >= SEN_TIMEVAL_STR_SIZE ? SEN_TIMEVAL_STR_SIZE - 1 : str_len;
+  memcpy(buf, str, len);
+  buf[len] = '\0';
+  memset(&tm, 0, sizeof(struct tm));
+  if (!format) { format = SEN_TIMEVAL_STRP_FORMAT; }
+  if (!(r1 = strptime(buf, format, &tm))) {
+    if ((format != SEN_TIMEVAL_STRP_FORMAT) ||
+        !(r1 = strptime(buf, "%Y/%m/%d %H:%M:%S", &tm))) {
+      return sen_invalid_argument;
+    }
+  }
+  if ((tv->tv_sec = (int32_t) mktime(&tm)) == -1) { return sen_invalid_argument; }
+  if (*r1 == '.') { r1++; }
+  uv = sen_atoi(r1, r1 + strlen(r1), &r2);
+  while (r2 < r1 + 6) {
+    uv *= 10;
+    r2++;
+  }
+  if (uv >= 1000000) { return sen_invalid_argument; }
+  tv->tv_usec = uv;
+  return sen_success;
+}
+
 #ifdef USE_FAIL_MALLOC
 double sen_fmalloc_prob = 1.0;
 char *sen_fmalloc_func = NULL;
@@ -415,11 +454,13 @@ sen_ctx_info_get(sen_ctx *ctx, sen_ctx_info *info)
     info->com_status = ctx->com_status;
     info->com_info = ctx->com_info;
     info->outbuf = &ctx->com->msg;
+    info->stat = ctx->stat;
   } else {
     info->fd = -1;
     info->com_status = 0;
     info->com_info = 0;
     info->outbuf = &ctx->outbuf;
+    info->stat = ctx->stat;
   }
   return sen_success;
 }
@@ -658,6 +699,25 @@ sen_obj_alloc(sen_ctx *ctx, uint32_t size)
   return NULL;
 }
 
+char *
+sen_obj_copy_bulk_value(sen_ctx *ctx, sen_obj *o)
+{
+  if (o->type != sen_ql_bulk) { return NULL; }
+  if (o->flags & SEN_OBJ_ALLOCATED) {
+    o->flags &= ~SEN_OBJ_ALLOCATED;
+    return o->u.b.value;
+  } else {
+    char *v = SEN_MALLOC(o->u.b.size + 1);
+    if (!v) {
+      MERR("malloc(%d) failed", o->u.b.size + 1);
+      return NULL;
+    }
+    memcpy(v, o->u.b.value, o->u.b.size);
+    v[o->u.b.size] = '\0';
+    return v;
+  }
+}
+
 void
 sen_obj_clear(sen_ctx *ctx, sen_obj *o)
 {
@@ -670,7 +730,13 @@ sen_obj_clear(sen_ctx *ctx, sen_obj *o)
       if (o->u.p.value) { sen_records_close(o->u.p.value); }
       break;
     case sen_ql_bulk :
-      if (o->u.b.value) { SEN_FREE(o->u.b.value); }
+      if (o->u.b.value) {
+        if (o->flags & SEN_OBJ_FROMJA) {
+          sen_ja_unref(ctx->db->values, 0, o->u.b.value, o->u.b.size);
+        } else {
+          SEN_FREE(o->u.b.value);
+        }
+      }
       break;
     case sen_ql_query :
       if (o->u.p.value) { sen_query_close(o->u.p.value); }
@@ -761,7 +827,7 @@ sen_logger_put(sen_log_level level,
     char lbuf[LBUFSIZE];
     if (sen_logger->flags & SEN_LOG_TIME) {
       sen_timeval tv;
-      if (sen_timeval_now(&tv) || sen_timeval_tostr(&tv, tbuf)) { tbuf[0] = '\0'; }
+      if (sen_timeval_now(&tv) || sen_timeval2str(&tv, tbuf)) { tbuf[0] = '\0'; }
     } else {
       tbuf[0] = '\0';
     }
