@@ -846,7 +846,7 @@ sen_index_info(sen_index *i, int *key_size, int *flags,
                int *initial_n_segments, sen_encoding *encoding,
                unsigned *nrecords_keys, unsigned *file_size_keys,
                unsigned *nrecords_lexicon, unsigned *file_size_lexicon,
-               unsigned *inv_seg_size, unsigned *inv_chunk_size)
+               unsigned long long *inv_seg_size, unsigned long long *inv_chunk_size)
 {
   sen_rc rc = sen_success;
 
@@ -862,16 +862,16 @@ sen_index_info(sen_index *i, int *key_size, int *flags,
     if ((rc = sen_sym_info(i->lexicon, NULL, NULL, NULL, nrecords_lexicon, file_size_lexicon))) { return rc; }
   }
   if (inv_seg_size || inv_chunk_size) {
-    off_t seg_size, chunk_size;
+    uint64_t seg_size, chunk_size;
 
     rc = sen_inv_info(i->inv, &seg_size, &chunk_size);
 
     if (inv_seg_size) {
-      *inv_seg_size = (unsigned)seg_size; /* XXX: inappropriate cast */
+      *inv_seg_size = seg_size;
     }
 
     if (inv_chunk_size) {
-      *inv_chunk_size = (unsigned)chunk_size; /* XXX: inappropriate cast */
+      *inv_chunk_size = chunk_size;
     }
 
     if (rc != sen_success) {
