@@ -1363,7 +1363,7 @@ sen_inv_close(sen_inv *inv)
 }
 
 sen_rc
-sen_inv_info(sen_inv *inv, off_t *seg_size, off_t *chunk_size)
+sen_inv_info(sen_inv *inv, uint64_t *seg_size, uint64_t *chunk_size)
 {
   sen_rc rc;
 
