@@ -134,9 +134,11 @@ sen_io_create(const char *path, uint32_t header_size, uint32_t segment_size,
             io->base_seg = bs;
             io->mode = mode;
             io->cache_size = cache_size;
+            io->header->curr_size = b;
             io->fis = fis;
             io->nmaps = 0;
             io->count = 0;
+            io->v08p = 0;
             return io;
           }
           SEN_GFREE(io);
@@ -213,6 +215,7 @@ sen_io_open(const char *path, sen_io_mode mode, uint32_t cache_size)
             io->fis = fis;
             io->nmaps = 0;
             io->count = 0;
+            io->v08p = v08p;
             return io;
           }
           SEN_GFREE(io->maps);
@@ -285,19 +288,23 @@ gen_pathname(const char *path, char *buffer, int fno)
 }
 
 sen_rc
-sen_io_size(sen_io *io, off_t *size)
+sen_io_size(sen_io *io, uint64_t *size)
 {
   int fno;
   struct stat s;
-  off_t tsize = 0;
+  uint64_t tsize = 0;
   char buffer[PATH_MAX];
-  uint32_t bs = io->base_seg;
-  uint32_t max_segment = io->header->max_segment;
-  uint32_t segment_size = io->header->segment_size;
-  unsigned int max_nfiles = (unsigned int)(
-    ((uint64_t)segment_size * (max_segment + bs) + SEN_IO_FILE_SIZE - 1)
-    / SEN_IO_FILE_SIZE);
-  for (fno = 0; fno < max_nfiles; fno++) {
+  uint32_t nfiles;
+  if (io->header->curr_size) {
+    nfiles = (uint32_t) ((io->header->curr_size + SEN_IO_FILE_SIZE - 1) / SEN_IO_FILE_SIZE);
+  } else {
+    uint32_t bs = io->base_seg;
+    uint32_t max_segment = io->header->max_segment;
+    uint32_t segment_size = io->header->segment_size;
+    nfiles = (uint32_t) (((uint64_t)segment_size * (max_segment + bs) + SEN_IO_FILE_SIZE - 1)
+                    / SEN_IO_FILE_SIZE);
+  }
+  for (fno = 0; fno < nfiles; fno++) {
     gen_pathname(io->path, buffer, fno);
     if (stat(buffer, &s)) {
       GSERR(buffer);
@@ -420,6 +427,10 @@ sen_io_win_map(sen_io *io, sen_ctx *ctx, sen_io_win *iw, uint32_t segment,
       if (!(p = SEN_MMAP(&iw->fmo, fi, pos, nseg * segment_size))) {
         return NULL;
       }
+      if (!io->v08p) {
+        uint64_t tail = io->base + segment * segment_size + offset + size;
+        if (tail > io->header->curr_size) { io->header->curr_size = tail; }
+      }
     } else {
       SEN_LOG(sen_log_alert, "nseg == 0! in sen_io_win_map(%p, %u, %u, %u)", io, segment, offset, size);
       return NULL;
@@ -430,6 +441,7 @@ sen_io_win_map(sen_io *io, sen_ctx *ctx, sen_io_win *iw, uint32_t segment,
   case sen_io_wronly:
     if (!(p = SEN_MALLOC(size))) { return NULL; }
     memset(p, 0, size);
+    iw->cached = 0;
     iw->addr = p;
     break;
   default :
@@ -498,6 +510,7 @@ retry:
         {
             /* AIO + DIO + cache hack */
             /* calc alignment */
+            // todo : calculate curr_size.
             off_t voffset = pos - (pos % MEM_ALIGN);
             uint32_t vsize = pos + size;
 
@@ -649,7 +662,11 @@ sen_io_win_unmap(sen_io_win *iw)
       }
       if (!rc) {
         if (!(rc = sen_pwrite(fi, iw->addr, iw->size, iw->pos))) {
-          SEN_FREE(iw->addr);
+          if (!io->v08p) {
+            uint64_t tail = io->base + iw->segment * segment_size + iw->offset + iw->size;
+            if (tail > io->header->curr_size) { io->header->curr_size = tail; }
+          }
+          if (!iw->cached) { SEN_FREE(iw->addr); }
           iw->addr = NULL;
         }
 #ifdef USE_AIO
@@ -682,12 +699,20 @@ sen_io_win_unmap(sen_io_win *iw)
       if ((info->map = SEN_MMAP(&info->fmo, fi, pos, segment_size))) {\
         uint32_t nmaps;\
         SEN_ATOMIC_ADD_EX(&io->nmaps, 1, nmaps);\
+        if (!io->v08p) {\
+          uint64_t tail = io->base + (segno + 1) * segment_size;\
+          if (tail > io->header->curr_size) { io->header->curr_size = tail; }\
+        }\
       }\
     }\
   } else {\
     if ((info->map = SEN_MMAP(&info->fmo, fi, pos, segment_size))) {\
       uint32_t nmaps;\
       SEN_ATOMIC_ADD_EX(&io->nmaps, 1, nmaps);\
+      if (!io->v08p) {\
+        uint64_t tail = io->base + (segno + 1) * segment_size;\
+        if (tail > io->header->curr_size) { io->header->curr_size = tail; }\
+      }\
     }\
   }\
 }
