@@ -91,6 +91,7 @@ struct _sen_io {
   struct _sen_io_fileinfo *fis;
   uint32_t nmaps;
   uint32_t count;
+  uint8_t v08p;
 };
 
 sen_io *sen_io_create(const char *path, uint32_t header_size, uint32_t segment_size,
@@ -98,7 +99,7 @@ sen_io *sen_io_create(const char *path, uint32_t header_size, uint32_t segment_s
 sen_io *sen_io_open(const char *path, sen_io_mode mode, uint32_t cache_size);
 sen_rc sen_io_close(sen_io *io);
 sen_rc sen_io_remove(const char *path);
-sen_rc sen_io_size(sen_io *io, off_t *size);
+sen_rc sen_io_size(sen_io *io, uint64_t *size);
 sen_rc sen_io_rename(const char *old_name, const char *new_name);
 void *sen_io_header(sen_io *io);
 
