@@ -35,9 +35,9 @@ nnn  This library is distributed in the hope that it will be useful,
 typedef sen_obj cell;
 
 struct _sen_query {
-  const char *str;
-  const char *cur;
-  const char *str_end;
+  char *str;
+  char *cur;
+  char *str_end;
   sen_sel_operator default_op;
   sen_select_optarg opt;
   sen_sel_mode default_mode;
@@ -135,9 +135,8 @@ skip_space(sen_query *q)
 inline static cell *
 get_phrase(sen_query *q)
 {
-  char *d = (char *)q->cur;
-  const char *start, *s;
-  start = s = q->cur;
+  char *start, *s, *d;
+  start = s = d = q->cur;
   while (1) {
     unsigned int len;
     if (s >= q->str_end) {
@@ -162,7 +161,7 @@ get_phrase(sen_query *q)
 inline static cell *
 get_word(sen_query *q, int *prefixp)
 {
-  const char *start = q->cur, *end;
+  char *start = q->cur, *end;
   unsigned int len;
   for (end = q->cur;; ) {
     /* null check and length check */
@@ -188,27 +187,27 @@ get_word(sen_query *q, int *prefixp)
 inline static cell *
 get_op(sen_query *q, sen_sel_operator op, int weight)
 {
-  const char *start, *end = q->cur;
+  char *start, *end = q->cur;
   int mode, option;
   switch (*end) {
   case 'S' :
     mode = sen_sel_similar;
     start = ++end;
-    option = sen_atoi(start, q->str_end, &end);
+    option = sen_atoi(start, q->str_end, (const char **)&end);
     if (start == end) { option = DEFAULT_SIMILARITY_THRESHOLD; }
     q->cur = end;
     break;
   case 'N' :
     mode = sen_sel_near;
     start = ++end;
-    option = sen_atoi(start, q->str_end, &end);
+    option = sen_atoi(start, q->str_end, (const char **)&end);
     if (start == end) { option = DEFAULT_MAX_INTERVAL; }
     q->cur = end;
     break;
   case 'T' :
     mode = sen_sel_term_extract;
     start = ++end;
-    option = sen_atoi(start, q->str_end, &end);
+    option = sen_atoi(start, q->str_end, (const char **)&end);
     if (start == end) { option = DEFAULT_TERM_EXTRACT_POLICY; }
     q->cur = end;
     break;
@@ -349,17 +348,17 @@ get_weight_vector(sen_query *query, const char *source)
 inline static void
 get_pragma(sen_query *q)
 {
-  const char *start, *end = q->cur;
+  char *start, *end = q->cur;
   while (end < q->str_end && *end == SEN_QUERY_PREFIX) {
     if (++end >= q->str_end) { break; }
     switch (*end) {
     case 'E' :
       start = ++end;
-      q->escalation_threshold = sen_atoi(start, q->str_end, &end);
+      q->escalation_threshold = sen_atoi(start, q->str_end, (const char **)&end);
       while (end < q->str_end && (isdigit(*end) || *end == '-')) { end++; }
       if (*end == ',') {
         start = ++end;
-        q->escalation_decaystep = sen_atoi(start, q->str_end, &end);
+        q->escalation_decaystep = sen_atoi(start, q->str_end, (const char **)&end);
       }
       q->cur = end;
       break;
@@ -388,7 +387,7 @@ get_pragma(sen_query *q)
       break;
     case 'W' :
       start = ++end;
-      end = get_weight_vector(q, start);
+      end = (char *)get_weight_vector(q, start);
       q->cur = end;
       break;
     }
@@ -414,13 +413,15 @@ sen_query_open(const char *str, unsigned int str_len,
   sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
   sen_query *q;
   int max_cells = max_exprs * 4;
-  if (!(q = SEN_MALLOC(sizeof(sen_query) + max_cells * sizeof(cell)))) {
+  if (!(q = SEN_MALLOC(sizeof(sen_query) + max_cells * sizeof(cell) + str_len + 1))) {
     SEN_LOG(sen_log_alert, "sen_query_open malloc fail");
     return NULL;
   }
-  q->str = str;
-  q->cur = str;
-  q->str_end = str + str_len;
+  q->str = (char *)&q->cell_pool[max_cells];
+  memcpy(q->str, str, str_len);
+  q->str[str_len] = '\0';
+  q->cur = q->str;
+  q->str_end = q->str + str_len;
   q->default_op = default_op;
   q->encoding = encoding;
   q->max_exprs = max_exprs;
@@ -486,8 +487,12 @@ exec_query(sen_index *i, sen_query *q, cell *c, sen_records *r, sen_sel_operator
     POP(e, c);
     switch (e->type) {
     case sen_ql_op :
-      ope = e;
-      op1 = ope->u.op.op;
+      if (opp == &op0 && e->u.op.op == sen_sel_but) {
+        POP(e, c);
+      } else {
+        ope = e;
+        op1 = ope->u.op.op;
+      }
       continue;
     case sen_ql_bulk :
       if (ope != NIL) {
@@ -516,7 +521,7 @@ exec_query(sen_index *i, sen_query *q, cell *c, sen_records *r, sen_sel_operator
       exec_query(i, q, e, s, *opp);
       break;
     default :
-      SEN_LOG(sen_log_notice, "invalid object assigned in query");
+      SEN_LOG(sen_log_notice, "invalid object assigned in query (%d)", e->type);
       break;
     }
     opp = &op1;
@@ -549,7 +554,7 @@ sen_rc
 sen_query_exec(sen_index *i, sen_query *q, sen_records *r, sen_sel_operator op)
 {
   int p;
-  if (!i || !q || !r || !LISTP(q->expr)) { return sen_invalid_argument; }
+  if (!i || !q || !r || !PAIRP(q->expr)) { return sen_invalid_argument; }
   p = q->escalation_threshold;
   // dump_query(q, q->expr, 0);
   // sen_log("escalation_threshold=%d", p);
@@ -580,7 +585,7 @@ query_term_rec(sen_query* q, cell* c, query_term_callback func, void *func_arg)
   if (BULKP(c)) {
     return func(c->u.b.value, c->u.b.size, func_arg);
   }
-  for (token = c; LISTP(token); token = CDR(token)) {
+  for (token = c; PAIRP(token); token = CDR(token)) {
     if (!query_term_rec(q, CAR(token), func, func_arg)) {
       return 0; /* abort */
     }
@@ -657,8 +662,12 @@ scan_query(sen_query *q, sen_nstr *nstr, sen_id section, cell *c, snip_cond **sc
     POP(e, c);
     switch (e->type) {
     case sen_ql_op :
-      ope = e;
-      op1 = ope->u.op.op;
+      if (opp == &op0 && e->u.op.op == sen_sel_but) {
+        POP(e, c);
+      } else {
+        ope = e;
+        op1 = ope->u.op.op;
+      }
       continue;
     case sen_ql_bulk :
       if (ope != NIL) {
@@ -692,7 +701,7 @@ scan_query(sen_query *q, sen_nstr *nstr, sen_id section, cell *c, snip_cond **sc
       scan_query(q, nstr, section, e, sc, *opp, flags, &_found, &_score);
       break;
     default :
-      SEN_LOG(sen_log_notice, "invalid object assigned in query");
+      SEN_LOG(sen_log_notice, "invalid object assigned in query! (%d)", e->type);
       break;
     }
     opp = &op1;
@@ -800,12 +809,11 @@ snip_query(sen_query *q, sen_snip *snip, cell *c, sen_sel_operator op,
       }
       break;
     case sen_ql_list :
-      if (*opp == sen_sel_but) { c_but ^= 1; }
-      snip_query(q, snip, e, *opp, n_tags, c_but,
+      snip_query(q, snip, e, *opp, n_tags, (*opp == sen_sel_but) ? c_but ^ 1 : c_but,
                  opentags, opentag_lens, closetags, closetag_lens);
       break;
     default :
-      SEN_LOG(sen_log_notice, "invalid object assigned in query");
+      SEN_LOG(sen_log_notice, "invalid object assigned in query!! (%d)", e->type);
       break;
     }
     opp = &op1;
