@@ -67,7 +67,7 @@ inline static void
 obj_ref(sen_obj *o)
 {
   if (o->nrefs < 0xffff) { o->nrefs++; }
-  if (LISTP(o)) { // todo : check cycle
+  if (PAIRP(o)) { // todo : check cycle
     if (CAR(o) != NIL) { obj_ref(CAR(o)); }
     if (CDR(o) != NIL) { obj_ref(CDR(o)); }
   }
@@ -81,7 +81,7 @@ obj_unref(sen_obj *o)
     return;
   }
   if (o->nrefs < 0xffff) { o->nrefs--; }
-  if (LISTP(o)) { // todo : check cycle
+  if (PAIRP(o)) { // todo : check cycle
     if (CAR(o) != NIL) { obj_unref(CAR(o)); }
     if (CDR(o) != NIL) { obj_unref(CDR(o)); }
   }
@@ -604,7 +604,7 @@ mk_atom(sen_ctx *ctx, char *str, unsigned int len, cell *v)
 inline static cell *
 mk_const(sen_ctx *ctx, char *name, unsigned int len)
 {
-  long    x;
+  int64_t x;
   char    tmp[256];
   char    tmp2[256];
   /* todo : rewirte with sen_str_* functions */
@@ -648,17 +648,17 @@ mk_const(sen_ctx *ctx, char *name, unsigned int len)
       memcpy(tmp2, name + 1, len);
       tmp2[len] = '\0';
       sprintf(tmp, "0%s", tmp2);
-      sscanf(tmp, "%lo", &x);
+      sscanf(tmp, "%Lo", &x);
       return mk_number(ctx, x);
     } else if (*name == 'd') {  /* #d (decimal) */
-      sscanf(&name[1], "%ld", &x);
+      sscanf(&name[1], "%Ld", &x);
       return mk_number(ctx, x);
     } else if (*name == 'x') {  /* #x (hex) */
       len = (len > 255) ? 255 : len - 1;
       memcpy(tmp2, name + 1, len);
       tmp2[len] = '\0';
       sprintf(tmp, "0x%s", tmp2);
-      sscanf(tmp, "%lx", &x);
+      sscanf(tmp, "%Lx", &x);
       return mk_number(ctx, x);
     }
   }
@@ -810,7 +810,7 @@ token(sen_ctx *ctx)
 }
 
 /* ========== Routines for Printing ========== */
-#define  ok_abbrev(x)  (LISTP(x) && CDR(x) == NIL)
+#define  ok_abbrev(x)  (PAIRP(x) && CDR(x) == NIL)
 
 void
 sen_obj_inspect(sen_ctx *ctx, sen_obj *obj, sen_rbuf *buf, int flags)
@@ -937,7 +937,7 @@ sen_obj_inspect(sen_ctx *ctx, sen_obj *obj, sen_rbuf *buf, int flags)
         for (;;) {
           sen_obj_inspect(ctx, CAR(obj), buf, flags);
           if ((obj = CDR(obj)) && (obj != NIL)) {
-            if (LISTP(obj)) {
+            if (PAIRP(obj)) {
               SEN_RBUF_PUTC(buf, ' ');
             } else {
               SEN_RBUF_PUTS(buf, " . ");
@@ -997,7 +997,7 @@ inline static cell *
 reverse(sen_ctx *ctx, cell *a) /* a must be checked by gc */
 {
   cell *p = NIL;
-  for ( ; LISTP(a); a = CDR(a)) {
+  for ( ; PAIRP(a); a = CDR(a)) {
     p = CONS(CAR(a), p);
     if (ERRP(ctx, SEN_ERROR)) { return F; }
   }
@@ -1102,7 +1102,7 @@ eqv(sen_obj *a, sen_obj *b)
 static cell *
 list_deep_copy(sen_ctx *ctx, cell *c) {
   /* NOTE: only list is copied */
-  if (LISTP(c)) {
+  if (PAIRP(c)) {
     /* TODO: convert recursion to loop */
     return CONS(list_deep_copy(ctx, CAR(c)), list_deep_copy(ctx, CDR(c)));
   } else {
@@ -1114,9 +1114,9 @@ static void
 qquote_uquotelist(sen_ctx *ctx, cell *cl, cell *pcl, int level) {
   /* reverse list */
   cell *x, *y;
-  while (cl != NIL) {
+  while (PAIRP(cl)) {
     x = CAR(cl);
-    if (LISTP(x)) {
+    if (PAIRP(x)) {
       y = CAR(x);
       if (y == UNQUOTE) {
         if (level) {
@@ -1245,7 +1245,7 @@ opexe(sen_ctx *ctx)
         if (NATIVE_FUNCP(ctx->code)) { s_return(ctx, ctx->code); }
         QLERR("Unbounded variable %s", SYMNAME(ctx->code));
       }
-    } else if (LISTP(ctx->code)) {
+    } else if (PAIRP(ctx->code)) {
       if (SYNTAXP(x = CAR(ctx->code))) {  /* SYNTAX */
         ctx->code = CDR(ctx->code);
         s_goto(ctx, SYNTAXNUM(x));
@@ -1272,7 +1272,7 @@ opexe(sen_ctx *ctx)
 
   case OP_E1ARGS:  /* eval arguments */
     ctx->args = CONS(ctx->value, ctx->args);
-    if (LISTP(ctx->code)) {  /* continue */
+    if (PAIRP(ctx->code)) {  /* continue */
       s_save(ctx, OP_E1ARGS, ctx->args, CDR(ctx->code));
       ctx->code = CAR(ctx->code);
       ctx->args = NIL;
@@ -1294,7 +1294,7 @@ opexe(sen_ctx *ctx)
       /* make environment */
       ctx->envir = CONS(NIL, CLOSURE_ENV(ctx->code));
       for (x = CAR(CLOSURE_CODE(ctx->code)), y = ctx->args;
-           LISTP(x); x = CDR(x), y = CDR(y)) {
+           PAIRP(x); x = CDR(x), y = CDR(y)) {
         if (y == NIL) {
           QLERR("Few arguments");
         } else {
@@ -1333,7 +1333,7 @@ opexe(sen_ctx *ctx)
     s_return(ctx, CAR(ctx->code));
 
   case OP_DEF0:  /* define */
-    if (LISTP(CAR(ctx->code))) {
+    if (PAIRP(CAR(ctx->code))) {
       x = CAAR(ctx->code);
       ctx->code = CONS(LAMBDA, CONS(CDAR(ctx->code), CDR(ctx->code)));
     } else {
@@ -1377,7 +1377,7 @@ opexe(sen_ctx *ctx)
     }
 
   case OP_BEGIN:    /* begin */
-    if (!LISTP(ctx->code)) {
+    if (!PAIRP(ctx->code)) {
       s_return(ctx, ctx->code);
     }
     if (CDR(ctx->code) != NIL) {
@@ -1407,7 +1407,8 @@ opexe(sen_ctx *ctx)
 
   case OP_LET1:    /* let (caluculate parameters) */
     ctx->args = CONS(ctx->value, ctx->args);
-    if (LISTP(ctx->code)) {  /* continue */
+    if (PAIRP(ctx->code)) {  /* continue */
+      QLASSERT(LISTP(CAR(ctx->code)) && LISTP(CDAR(ctx->code)));
       s_save(ctx, OP_LET1, ctx->args, CDR(ctx->code));
       ctx->code = CADAR(ctx->code);
       ctx->args = NIL;
@@ -1425,7 +1426,7 @@ opexe(sen_ctx *ctx)
          y != NIL; x = CDR(x), y = CDR(y))
       CAR(ctx->envir) = CONS(CONS(CAAR(x), CAR(y)), CAR(ctx->envir));
     if (SYMBOLP(CAR(ctx->code))) {  /* named let */
-      for (x = CADR(ctx->code), ctx->args = NIL; LISTP(x); x = CDR(x))
+      for (x = CADR(ctx->code), ctx->args = NIL; PAIRP(x); x = CDR(x))
         ctx->args = CONS(CAAR(x), ctx->args);
       x = mk_closure(ctx, CONS(non_alloc_rev(NIL, ctx->args), CDDR(ctx->code)),
                      ctx->envir);
@@ -1445,6 +1446,8 @@ opexe(sen_ctx *ctx)
       s_goto(ctx, OP_BEGIN);
     }
     s_save(ctx, OP_LET1AST, CDR(ctx->code), CAR(ctx->code));
+    QLASSERT(LISTP(CAR(ctx->code)) &&
+             LISTP(CAAR(ctx->code)) && LISTP((CDR(CAAR(ctx->code)))));
     ctx->code = CADAAR(ctx->code);
     s_goto(ctx, OP_EVAL);
 
@@ -1455,7 +1458,8 @@ opexe(sen_ctx *ctx)
   case OP_LET2AST:  /* let* (caluculate parameters) */
     CAR(ctx->envir) = CONS(CONS(CAAR(ctx->code), ctx->value), CAR(ctx->envir));
     ctx->code = CDR(ctx->code);
-    if (LISTP(ctx->code)) {  /* continue */
+    if (PAIRP(ctx->code)) {  /* continue */
+      QLASSERT(LISTP(CAR(ctx->code)) && LISTP(CDAR(ctx->code)));
       s_save(ctx, OP_LET2AST, ctx->args, ctx->code);
       ctx->code = CADAR(ctx->code);
       ctx->args = NIL;
@@ -1475,7 +1479,8 @@ opexe(sen_ctx *ctx)
 
   case OP_LET1REC:  /* letrec (caluculate parameters) */
     ctx->args = CONS(ctx->value, ctx->args);
-    if (LISTP(ctx->code)) {  /* continue */
+    if (PAIRP(ctx->code)) {  /* continue */
+      QLASSERT(LISTP(CAR(ctx->code)) && LISTP(CDAR(ctx->code)));
       s_save(ctx, OP_LET1REC, ctx->args, CDR(ctx->code));
       ctx->code = CADAR(ctx->code);
       ctx->args = NIL;
@@ -1495,7 +1500,7 @@ opexe(sen_ctx *ctx)
     s_goto(ctx, OP_BEGIN);
 
   case OP_COND0:    /* cond */
-    if (!LISTP(ctx->code)) {
+    if (!PAIRP(ctx->code)) {
       QLERR("Syntax error in cond");
     }
     s_save(ctx, OP_COND1, NIL, ctx->code);
@@ -1601,7 +1606,7 @@ opexe(sen_ctx *ctx)
 
   case OP_CASE1:    /* case */
     for (x = ctx->code; x != NIL; x = CDR(x)) {
-      if (!LISTP(y = CAAR(x)))
+      if (!PAIRP(y = CAAR(x)))
         break;
       for ( ; y != NIL; y = CDR(y))
         if (eqv(CAR(y), ctx->value))
@@ -1610,7 +1615,7 @@ opexe(sen_ctx *ctx)
         break;
     }
     if (x != NIL) {
-      if (LISTP(CAAR(x))) {
+      if (PAIRP(CAAR(x))) {
         ctx->code = CDAR(x);
         s_goto(ctx, OP_BEGIN);
       } else {/* else */
@@ -1644,7 +1649,7 @@ opexe(sen_ctx *ctx)
     s_goto(ctx, OP_APPLY);
 
   case OP_SETCAR:  /* set-car! */
-    if (LISTP(CAR(ctx->args))) {
+    if (PAIRP(CAR(ctx->args))) {
       CAAR(ctx->args) = CADR(ctx->args);
       s_return(ctx, CAR(ctx->args));
     } else {
@@ -1652,7 +1657,7 @@ opexe(sen_ctx *ctx)
     }
 
   case OP_SETCDR:  /* set-cdr! */
-    if (LISTP(CAR(ctx->args))) {
+    if (PAIRP(CAR(ctx->args))) {
       CDAR(ctx->args) = CADR(ctx->args);
       s_return(ctx, CAR(ctx->args));
     } else {
@@ -1853,14 +1858,16 @@ opexe(sen_ctx *ctx)
     ctx->code = CDR(ctx->args);
     s_goto(ctx, OP_QQUOTE1);
   case OP_QQUOTE1:
-    while (ctx->code != NIL) {
+    while (PAIRP(ctx->code)) {
       x = CAR(ctx->code);
-      if (LISTP(x)) {
+      if (PAIRP(x) && LISTP(CDR(x))) {
         s_save(ctx, OP_QQUOTE2, ctx->args, ctx->code);
         y = CADR(x);
         if (y == UNQUOTE) {
+          QLASSERT(LISTP(CDDR(x)));
           ctx->code = CADDR(x);
         } else if (CAR(y) == UNQUOTESP) {
+          QLASSERT(LISTP(CDR(y)));
           ctx->code = CADR(y);
         } else {
           y = CAR(x);
@@ -1885,7 +1892,7 @@ opexe(sen_ctx *ctx)
     } else if (CAR(y) == UNQUOTESP) {
       if (ctx->value == NIL) {
         CDR(x) = CDDR(x);
-      } else if (!LISTP(ctx->value) ) {
+      } else if (!PAIRP(ctx->value) ) {
         /* error */
       } else {
         ctx->value = list_deep_copy(ctx, ctx->value);
@@ -1900,7 +1907,7 @@ opexe(sen_ctx *ctx)
       } else if (CAR(y) == UNQUOTESP) {
         if (ctx->value == NIL) {
           CAR(x) = CDAR(x);
-        } else if (!LISTP(ctx->value) ) {
+        } else if (!PAIRP(ctx->value) ) {
           /* error */
         } else {
           ctx->value = list_deep_copy(ctx, ctx->value);
@@ -1953,14 +1960,21 @@ sen_ql_eval(sen_ctx *ctx, sen_obj *code, sen_obj *objs)
   sen_ql_co co;
   uint8_t op = ctx->op;
   uint8_t stat = ctx->stat;
+  uint8_t feed_mode = ctx->feed_mode;
+  sen_obj *o, *code_ = ctx->code;
+  o = CONS(objs, ctx->envir);
   memcpy(&co, &ctx->co, sizeof(sen_ql_co));
-  s_save(ctx, OP_QUIT, ctx->args, objs);
+  s_save(ctx, OP_QUIT, ctx->args, o);
   ctx->op = OP_EVAL;
   ctx->stat = SEN_QL_EVAL;
   ctx->code = code;
+  ctx->feed_mode = sen_ql_atonce;
   sen_ql_feed(ctx, NULL, 0, 0);
+  ctx->feed_mode = feed_mode;
   ctx->stat = stat;
   ctx->op = op;
+  ctx->envir = CDR(o);
+  ctx->code = code_;
   memcpy(&ctx->co, &co, sizeof(sen_ql_co));
   return ctx->value;
 }
@@ -2084,7 +2098,7 @@ sen_ql_eval(sen_ctx *ctx, sen_obj *code, sen_obj *objs)
   case sen_ql_float :\
     {\
       sen_timeval tv;\
-      double sec = x->u.tv.tv_sec op FVALUE(y);\
+      double sec = x->u.tv.tv_sec op (int)FVALUE(y);\
       int32_t usec = x->u.tv.tv_usec op (int)((FVALUE(y) - (int)FVALUE(y)) * 1000000);\
       if (sec < INT32_MIN || INT32_MAX < sec) { QLERR("time val overflow"); }\
       tv.tv_sec = (int)sec;\
@@ -2109,13 +2123,13 @@ static sen_obj *
 nf_add(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 {
   register cell *x, *v;
-  if (!LISTP(args)) { QLERR("list expected"); }
+  if (!PAIRP(args)) { QLERR("list expected"); }
   switch (CAR(args)->type) {
   case sen_ql_bulk :
     {
       sen_rbuf buf;
       sen_rbuf_init(&buf, 0);
-      while (LISTP(args)) {
+      while (PAIRP(args)) {
         POP(x, args);
         sen_obj_inspect(ctx, x, &buf, 0);
       }
@@ -2123,7 +2137,7 @@ nf_add(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
     }
     break;
   case sen_ql_time :
-    if (LISTP(CDR(args)) && NUMBERP(CADR(args))) {
+    if (PAIRP(CDR(args)) && NUMBERP(CADR(args))) {
       SEN_OBJ_NEW(ctx, v);
       time_op(CAR(args), CADR(args), v, +);
     } else {
@@ -2132,7 +2146,7 @@ nf_add(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
     break;
   default :
     v = mk_number(ctx, 0);
-    while (LISTP(args)) {
+    while (PAIRP(args)) {
       POP(x, args);
       do_op(v, x, +);
     }
@@ -2146,7 +2160,7 @@ nf_sub(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 {
   register cell *v = mk_number(ctx, 0);
   register cell *x;
-  if (LISTP(args) && CDR(args) != NIL) {
+  if (PAIRP(args) && CDR(args) != NIL) {
     if (CAR(args)->type == sen_ql_time) {
       time_op(CAR(args), CADR(args), v, -);
       return v;
@@ -2154,7 +2168,7 @@ nf_sub(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
     POP(x, args);
     do_op(v, x, +);
   }
-  while (LISTP(args)) {
+  while (PAIRP(args)) {
     POP(x, args);
     do_op(v, x, -);
   }
@@ -2176,7 +2190,7 @@ nf_mul(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
     SEN_RBUF2OBJ(ctx, &buf, v);
   } else {
     v = mk_number(ctx, 1);
-    while (LISTP(args)) {
+    while (PAIRP(args)) {
       POP(x, args);
       do_op(v, x, *);
     }
@@ -2189,14 +2203,14 @@ nf_div(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 {
   register cell *v;
   register cell *x;
-  if (LISTP(args) && CDR(args) != NIL) {
+  if (PAIRP(args) && CDR(args) != NIL) {
     v = mk_number(ctx, 0);
     POP(x, args);
     do_op(v, x, +);
   } else {
     v = mk_number(ctx, 1);
   }
-  while (LISTP(args)) {
+  while (PAIRP(args)) {
     POP(x, args);
     if (x->type == sen_ql_int && IVALUE(x) == 0 && v->type == sen_ql_int) {
       SETFLOAT(v, (double)IVALUE(v));
@@ -2231,7 +2245,7 @@ nf_rem(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 static sen_obj *
 nf_car(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 {
-  if (LISTP(CAR(args))) {
+  if (PAIRP(CAR(args))) {
     return CAAR(args);
   } else {
     QLERR("Unable to car for non-cons cell");
@@ -2240,7 +2254,7 @@ nf_car(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 static sen_obj *
 nf_cdr(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 {
-  if (LISTP(CAR(args))) {
+  if (PAIRP(CAR(args))) {
     return CDAR(args);
   } else {
     QLERR("Unable to cdr for non-cons cell");
@@ -2373,7 +2387,7 @@ nf_proc(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 static sen_obj *
 nf_pair(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 {
-  s_retbool(LISTP(CAR(args)));
+  s_retbool(PAIRP(CAR(args)));
 }
 static sen_obj *
 nf_eq(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
@@ -2440,7 +2454,7 @@ nf_length(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 {
   register long v;
   register cell *x;
-  for (x = CAR(args), v = 0; LISTP(x); x = CDR(x)) { ++v; }
+  for (x = CAR(args), v = 0; PAIRP(x); x = CDR(x)) { ++v; }
   return mk_number(ctx, v);
 }
 static sen_obj *
@@ -2448,14 +2462,14 @@ nf_assq(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 {
   register cell *x, *y;
   x = CAR(args);
-  for (y = CADR(args); LISTP(y); y = CDR(y)) {
-    if (!LISTP(CAR(y))) {
+  for (y = CADR(args); PAIRP(y); y = CDR(y)) {
+    if (!PAIRP(CAR(y))) {
       QLERR("Unable to handle non pair element");
     }
     if (x == CAAR(y))
       break;
   }
-  if (LISTP(y)) {
+  if (PAIRP(y)) {
     return CAR(y);
   } else {
     return F;
@@ -2503,7 +2517,7 @@ nf_voidp(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 static sen_obj *
 nf_list(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 {
-  if (LISTP(args)) {
+  if (PAIRP(args)) {
     return args;
   } else {
     QLERR("Unable to handle non-cons argument");
@@ -2566,14 +2580,14 @@ nf_timestr(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
   default :
     QLERR("can't convert into time");
   }
-  if (sen_timeval_tostr(&tv, buf)) { QLERR("timeval_tostr failed"); }
+  if (sen_timeval2str(&tv, buf)) { QLERR("timeval2str failed"); }
   return sen_ql_mk_string(ctx, buf, strlen(buf));
 }
 static sen_obj *
 nf_tonumber(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
 {
   sen_obj *x, *v;
-  if (!LISTP(args)) { QLERR("list expected"); }
+  if (!PAIRP(args)) { QLERR("list expected"); }
   x = CAR(args);
   switch (x->type) {
   case sen_ql_bulk :
@@ -2592,7 +2606,46 @@ nf_tonumber(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
     }
     break;
   default :
-    QLERR("can't convert into time");
+    QLERR("can't convert into number");
+  }
+  return v;
+}
+static sen_obj *
+nf_totime(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
+{
+  sen_timeval tv;
+  sen_obj *x, *v;
+  if (!PAIRP(args)) { QLERR("list expected"); }
+  x = CAR(args);
+  switch (x->type) {
+  case sen_ql_bulk :
+    {
+      const char *fmt = NULL;
+      if (PAIRP(CDR(args)) && BULKP(CADR(args))) { fmt = STRVALUE(CADR(args)); }
+      if (sen_str2timeval(STRVALUE(x), x->u.b.size, fmt, &tv)) {
+        QLERR("cast error");
+      }
+      SEN_OBJ_NEW(ctx, v);
+      SETTIME(v, &tv);
+    }
+    break;
+  case sen_ql_int :
+    tv.tv_sec = (int32_t) IVALUE(x);
+    tv.tv_usec = 0;
+    SEN_OBJ_NEW(ctx, v);
+    SETTIME(v, &tv);
+    break;
+  case sen_ql_float :
+    tv.tv_sec = (int32_t) FVALUE(x);
+    tv.tv_usec = (int32_t) ((FVALUE(x) - tv.tv_sec) * 1000000);
+    SEN_OBJ_NEW(ctx, v);
+    SETTIME(v, &tv);
+    break;
+  case sen_ql_time :
+    v = x;
+    break;
+  default :
+    QLERR("can't convert into number");
   }
   return v;
 }
@@ -2733,6 +2786,7 @@ init_procs(sen_ctx *ctx)
   sen_ql_def_native_func(ctx, "loglevel", nf_loglevel);
   sen_ql_def_native_func(ctx, "now", nf_now);
   sen_ql_def_native_func(ctx, "timestr", nf_timestr);
+  sen_ql_def_native_func(ctx, "x->time", nf_totime);
   sen_ql_def_native_func(ctx, "x->number", nf_tonumber);
 }
 
