@@ -293,8 +293,9 @@ sen_snip_add_cond(sen_snip *snip,
 {
   sen_rc rc;
   snip_cond *cond;
+  sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
 
-  if (!snip || !keyword || !*keyword || snip->cond_len >= MAX_SNIP_COND_COUNT) {
+  if (!snip || !keyword || !keyword_len || snip->cond_len >= MAX_SNIP_COND_COUNT) {
     return sen_invalid_argument;
   }
   cond = snip->cond + snip->cond_len;
@@ -307,14 +308,30 @@ sen_snip_add_cond(sen_snip *snip,
     return sen_invalid_argument;
   }
   if (opentag) {
-    cond->opentag = opentag;
+    if (snip->flags & SEN_SNIP_COPY_TAG) {
+      char *t = SEN_MALLOC(opentag_len + 1);
+      if (!t) { return sen_memory_exhausted; }
+      memcpy(t, opentag, opentag_len);
+      t[opentag_len]= '\0';
+      cond->opentag = t;
+    } else {
+      cond->opentag = opentag;
+    }
     cond->opentag_len = opentag_len;
   } else {
     cond->opentag = snip->defaultopentag;
     cond->opentag_len = snip->defaultopentag_len;
   }
   if (closetag) {
-    cond->closetag = closetag;
+    if (snip->flags & SEN_SNIP_COPY_TAG) {
+      char *t = SEN_MALLOC(closetag_len + 1);
+      if (!t) { return sen_memory_exhausted; }
+      memcpy(t, closetag, closetag_len);
+      t[closetag_len]= '\0';
+      cond->closetag = t;
+    } else {
+      cond->closetag = closetag;
+    }
     cond->closetag_len = closetag_len;
   } else {
     cond->closetag = snip->defaultclosetag;
@@ -403,8 +420,17 @@ sen_snip_close(sen_snip *snip)
 {
   sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
   snip_cond *cond, *cond_end;
-  if (!snip) {
-    return sen_invalid_argument;
+  if (!snip) { return sen_invalid_argument; }
+  if (snip->flags & SEN_SNIP_COPY_TAG) {
+    int i;
+    snip_cond *sc;
+    const char *dot = snip->defaultopentag, *dct = snip->defaultclosetag;
+    for (i = snip->cond_len, sc = snip->cond; i; i--, sc++) {
+      if (sc->opentag != dot) { SEN_FREE((void *)sc->opentag); }
+      if (sc->closetag != dct) { SEN_FREE((void *)sc->closetag); }
+    }
+    if (dot) { SEN_FREE((void *)dot); }
+    if (dct) { SEN_FREE((void *)dct); }
   }
   if (snip->nstr) {
     sen_nstr_close(snip->nstr);
@@ -422,7 +448,7 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
               unsigned int *nresults, unsigned int *max_tagged_len)
 {
   size_t i;
-  if (!snip || !string || !*string) {
+  if (!snip || !string) {
     return sen_invalid_argument;
   }
   exec_clean(snip);
