@@ -14,12 +14,16 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-#ifndef _SEN_SNIP_H
-#define _SEN_SNIP_H
+#ifndef SEN_SNIP_H
+#define SEN_SNIP_H
 
-#ifndef _SENNA_H
+#ifndef SENNA_H
 #include "senna_in.h"
-#endif /* _SENNA_H */
+#endif /* SENNA_H */
+
+#ifndef SEN_STR_H
+#include "str.h"
+#endif /* SEN_STR_H */
 
 #define ASIZE                   256U
 #define MAX_SNIP_TAG_COUNT      512U
@@ -31,8 +35,6 @@ extern "C"
 {
 #endif
 
-#include "str.h"
-
 #define SNIPCOND_NONSTOP 0
 #define SNIPCOND_STOP    1
 #define SNIPCOND_ACROSS  2
@@ -120,4 +122,4 @@ void sen_bm_tunedbm(snip_cond *cond, sen_nstr *object);
 }
 #endif
 
-#endif /* _SEN_SNIP_H */
+#endif /* SEN_SNIP_H */
