@@ -195,7 +195,8 @@ sen_ra_at(sen_ra *ra, sen_id id)
 {
   void *p;
   uint16_t seg;
-  if (id > ra->header->curr_max) { return NULL; }
+  static char buf[8] = {0, 0, 0, 0, 0, 0, 0, 0};
+  if (id > ra->header->curr_max) { return (void *) buf; }
   seg = id >> ra->element_width;
   SEN_IO_SEG_MAP(ra->io, seg, p);
   if (!p) { return NULL; }
@@ -204,6 +205,8 @@ sen_ra_at(sen_ra *ra, sen_id id)
 
 /**** jagged arrays ****/
 
+#define USE_SEG_MAP
+
 #define SEN_JA_IDSTR "SENNA:JA:01.000"
 
 #define W_OF_JA_MAX 38
@@ -353,48 +356,7 @@ sen_ja_remove(const char *path)
   return sen_io_remove(path);
 }
 
-sen_rc
-sen_ja_put(sen_ja *ja, sen_id id, const void *value, int value_len, int flags)
-{
-  int rc;
-  void *buf;
-  sen_ja_einfo einfo;
-  if ((flags & SEN_ST_APPEND)) {
-    uint32_t old_len;
-    const void *oldvalue = sen_ja_ref(ja, id, &old_len);
-    if (oldvalue) {
-      if ((rc = sen_ja_alloc(ja, value_len + old_len, &einfo, &buf))) { return rc; }
-      memcpy(buf, oldvalue, old_len);
-      memcpy((byte *)buf + old_len, value, value_len);
-      sen_ja_unref(ja, id);
-    } else {
-      if ((rc = sen_ja_alloc(ja, value_len, &einfo, &buf))) { return rc; }
-      memcpy(buf, value, value_len);
-    }
-  } else {
-    if (value_len) {
-      if ((rc = sen_ja_alloc(ja, value_len, &einfo, &buf))) { return rc; }
-      // printf("put id=%d, value_len=%d value=%p ei=%p(%d:%d)\n", id, value_len, buf, &einfo, einfo.u.s.pos, einfo.u.s.tail[0]);
-      memcpy(buf, value, value_len);
-    } else {
-      memset(&einfo, 0, sizeof(sen_ja_einfo));
-    }
-  }
-  return sen_ja_replace(ja, id, &einfo);
-}
-
-int
-sen_ja_at(sen_ja *ja, sen_id id, void *valbuf, int buf_size)
-{
-  uint32_t len;
-  const void *value = sen_ja_ref(ja, id, &len);
-  if (!value) { return -1; }
-  if (buf_size >= len) { memcpy(valbuf, value, len); }
-  sen_ja_unref(ja, id);
-  return (int) len;
-}
-
-const void *
+void *
 sen_ja_ref(sen_ja *ja, sen_id id, uint32_t *value_len)
 {
   sen_ja_einfo *einfo;
@@ -412,22 +374,100 @@ sen_ja_ref(sen_ja *ja, sen_id id, uint32_t *value_len)
   {
     void *value;
     uint32_t jag, vpos, vsize;
+#ifdef USE_SEG_MAP
     EINFO_GET(&einfo[pos], jag, vpos, vsize);
     SEN_IO_SEG_MAP(ja->io, jag, value);
     // printf("at id=%d value=%p jag=%d vpos=%d ei=%p(%d:%d)\n", id, value, jag, vpos, &einfo[pos], einfo[pos].u.s.pos, einfo[pos].u.s.tail[0]);
     if (!value) { *value_len = 0; return NULL; }
     *value_len = vsize;
     return (byte *)value + vpos;
+#else /* USE_SEG_MAP */
+    sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
+    sen_io_win iw;
+    EINFO_GET(&einfo[pos], jag, vpos, vsize);
+    value = sen_io_win_map(ja->io, ctx, &iw, jag, vpos, vsize, sen_io_rdonly);
+    if (!value) { *value_len = 0; return NULL; }
+    *value_len = vsize;
+    return value;
+#endif /* USE_SEG_MAP */
   }
 }
 
 sen_rc
-sen_ja_unref(sen_ja *ja, sen_id id)
+sen_ja_unref(sen_ja *ja, sen_id id, void *value, uint32_t value_len)
 {
-  // todo
+  if (!value) { return sen_invalid_argument; }
+#ifndef USE_SEG_MAP
+  {
+    sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
+    SEN_FREE(value);
+  }
+#endif /* USE_SEG_MAP */
   return sen_success;
 }
 
+sen_rc sen_ja_alloc(sen_ja *ja, int element_size, sen_ja_einfo *einfo, sen_io_win *iw);
+
+sen_rc
+sen_ja_put(sen_ja *ja, sen_id id, void *value, int value_len, int flags)
+{
+  int rc;
+  sen_io_win iw;
+  sen_ja_einfo einfo;
+  if ((flags & SEN_ST_APPEND)) {
+    if (value_len) {
+      uint32_t old_len;
+      void *oldvalue = sen_ja_ref(ja, id, &old_len);
+      if (oldvalue) {
+#ifdef USE_SEG_MAP
+        if ((rc = sen_ja_alloc(ja, value_len + old_len, &einfo, &iw))) { return rc; }
+        memcpy(iw.addr, oldvalue, old_len);
+        memcpy((byte *)iw.addr + old_len, value, value_len);
+#else /* USE_SEG_MAP */
+        void *buf;
+        sen_ctx *ctx = &sen_gctx; /* todo : replace it with the local ctx */
+        if (!(buf = SEN_MALLOC(old_len + value_len))) { return sen_memory_exhausted; }
+        if ((rc = sen_ja_alloc(ja, old_len + value_len, &einfo, &iw))) {
+          SEN_FREE(buf);
+          return rc;
+        }
+        memcpy(buf, oldvalue, old_len);
+        memcpy((byte *)buf + old_len, value, value_len);
+        iw.addr = buf;
+        sen_io_win_unmap(&iw);
+#endif /* USE_SEG_MAP */
+        sen_ja_unref(ja, id, oldvalue, old_len);
+      } else {
+#ifdef USE_SEG_MAP
+        if ((rc = sen_ja_alloc(ja, value_len, &einfo, &iw))) { return rc; }
+        memcpy(iw.addr, value, value_len);
+#else /* USE_SEG_MAP */
+        sen_io_win iw;
+        if ((rc = sen_ja_alloc(ja, value_len, &einfo, &iw))) { return rc; }
+        iw.addr = value;
+        sen_io_win_unmap(&iw);
+#endif /* USE_SEG_MAP */
+      }
+    }
+  } else {
+    if (value_len) {
+#ifdef USE_SEG_MAP
+      if ((rc = sen_ja_alloc(ja, value_len, &einfo, &iw))) { return rc; }
+      // printf("put id=%d, value_len=%d value=%p ei=%p(%d:%d)\n", id, value_len, buf, &einfo, einfo.u.s.pos, einfo.u.s.tail[0]);
+      memcpy(iw.addr, value, value_len);
+#else /* USE_SEG_MAP */
+      sen_io_win iw;
+      if ((rc = sen_ja_alloc(ja, value_len, &einfo, &iw))) { return rc; }
+      iw.addr = value;
+      sen_io_win_unmap(&iw);
+#endif /* USE_SEG_MAP */
+    } else {
+      memset(&einfo, 0, sizeof(sen_ja_einfo));
+    }
+  }
+  return sen_ja_replace(ja, id, &einfo);
+}
+
 int
 sen_ja_size(sen_ja *ja, sen_id id)
 {
@@ -447,16 +487,18 @@ sen_ja_size(sen_ja *ja, sen_id id)
 }
 
 sen_rc
-sen_ja_alloc(sen_ja *ja, int element_size, sen_ja_einfo *einfo, void **value)
+sen_ja_alloc(sen_ja *ja, int element_size, sen_ja_einfo *einfo, sen_io_win *iw)
 {
   int m, size;
-  void *addr;
   ja_pos *vp;
+#ifdef USE_SEG_MAP
+  void *addr;
   if (element_size < 8) {
     einfo->u.s.tail[1] = element_size * 2 + 1;
-    *value = (void *)einfo;
+    iw->addr = (void *)einfo;
     return sen_success;
   }
+#endif /* USE_SEG_MAP */
   if (element_size >= ja->header->max_element_size) {
     return sen_invalid_argument;
   }
@@ -472,10 +514,32 @@ sen_ja_alloc(sen_ja *ja, int element_size, sen_ja_einfo *einfo, void **value)
     vp->pos = 0;
   }
   EINFO_SET(einfo, vp->seg, vp->pos, element_size);
+#ifdef USE_SEG_MAP
   SEN_IO_SEG_MAP(ja->io, vp->seg, addr);
   // printf("addr=%p seg=%d pos=%d\n", addr, vp->seg, vp->pos);
   if (!addr) { return sen_memory_exhausted; }
-  *value = (byte *)addr + vp->pos;
+  iw->addr = (byte *)addr + vp->pos;
+#else /* USE_SEG_MAP */
+  iw->io = ja->io;
+  iw->ctx = &sen_gctx;
+  iw->mode = sen_io_wronly;
+  iw->segment = vp->seg;
+  iw->offset = vp->pos;
+  iw->size = element_size;
+  iw->cached = 1;
+  {
+    int fno, base;
+    uint32_t nseg, bseg;
+    uint32_t segment_size = JA_SEGMENT_SIZE;
+    uint32_t segments_per_file = SEN_IO_FILE_SIZE / segment_size;
+    nseg = (iw->offset + element_size + segment_size - 1) / segment_size;
+    bseg = iw->segment + ja->io->base_seg;
+    fno = bseg / segments_per_file;
+    base = fno ? 0 : ja->io->base - ja->io->base_seg * segment_size;
+    iw->pos = (bseg % segments_per_file) * segment_size + iw->offset + base;
+    iw->nseg = nseg;
+  }
+#endif /* USE_SEG_MAP */
   if ((vp->pos += size) == JA_SEGMENT_SIZE) {
     vp->seg = 0;
     vp->pos = 0;
@@ -579,15 +643,18 @@ sen_rc
 sen_db_idx_slot_build(sen_db *s, sen_db_store *store)
 {
   sen_index *index = store->u.i.index;
-  sen_id id = SEN_SYM_NIL, maxid = sen_sym_curr_id(index->keys);
+  sen_id id = SEN_SYM_NIL; /* maxid = sen_sym_curr_id(index->keys); */
   sen_db_store *target = sen_db_store_by_id(s, store->triggers->target);
   if (!target) { return sen_invalid_argument; }
-  while (++id <= maxid) {
+  while ((id = sen_sym_next(index->keys, id))) {
     uint32_t vs;
     const char *key = _sen_sym_key(index->keys, id);
     void *vp = (void *)sen_ja_ref(target->u.v.ja, id, &vs);
-    if (key && vp && vs && sen_index_upd(index, key, NULL, 0, vp, vs)) {
-      SEN_LOG(sen_log_error, "sen_index_upd failed. id=%d key=(%s)", id, key);
+    if (vp) {
+      if (key && vs && sen_index_upd(index, key, NULL, 0, vp, vs)) {
+        SEN_LOG(sen_log_error, "sen_index_upd failed. id=%d key=(%s)", id, key);
+      }
+      sen_ja_unref(target->u.v.ja, id, vp, vs);
     }
   }
   return sen_success;
@@ -600,7 +667,7 @@ sen_db_store_open(sen_db *s, const char *name)
   uint32_t spec_len;
   sen_db_store *e, *k, *l, *r = NULL;
   char buffer[PATH_MAX];
-  const sen_db_store_spec *spec;
+  sen_db_store_spec *spec;
   if (!s || !(id = sen_sym_at(s->keys, name))) {
     return NULL;
   }
@@ -613,7 +680,7 @@ sen_db_store_open(sen_db *s, const char *name)
   if (spec->type == sen_db_idx_slot) {
     if (!(k = sen_db_store_by_id(s, spec->u.s.class)) ||
         !(l = sen_db_slot_class(s, name))) {
-      sen_ja_unref(s->values, id);
+      sen_ja_unref(s->values, id, spec, spec_len);
       return NULL;
     }
   }
@@ -672,7 +739,7 @@ sen_db_store_open(sen_db *s, const char *name)
   e->type = spec->type;
   r = e;
 exit :
-  sen_ja_unref(s->values, id);
+  sen_ja_unref(s->values, id, spec, spec_len);
   MUTEX_UNLOCK(s->lock);
   return r;
 }
@@ -784,7 +851,7 @@ sen_db_store_add_trigger(sen_db_store *e, sen_db_store_rel_spec *t)
   sen_rc rc;
   sen_db *s = e->db;
   uint32_t spec_len, newspec_len;
-  const sen_db_store_spec *spec;
+  sen_db_store_spec *spec;
   sen_db_store_spec *newspec;
   sen_db_store *target = sen_db_store_by_id(s, t->target);
   if (!target) { return sen_invalid_argument; }
@@ -805,7 +872,7 @@ sen_db_store_add_trigger(sen_db_store *e, sen_db_store_rel_spec *t)
   memcpy(newspec, spec, spec_len);
   memcpy(&newspec->triggers[spec->n_triggers], t, sizeof(sen_db_store_rel_spec));
   newspec->n_triggers++;
-  sen_ja_unref(s->values, e->id);
+  sen_ja_unref(s->values, e->id, spec, spec_len);
   rc = sen_ja_put(s->values, e->id, newspec, newspec_len, 0);
   sen_db_unlock(s);
   SEN_GFREE(newspec);
@@ -837,7 +904,7 @@ sen_db_store_del_trigger(sen_db_store *e, sen_db_store_rel_spec *t)
   sen_db *s = e->db;
   uint32_t spec_len, newspec_len;
   uint32_t n_triggers = 0;
-  const sen_db_store_spec *spec;
+  sen_db_store_spec *spec;
   sen_db_store_spec *newspec;
   sen_db_trigger *r, **rp = &e->triggers;
   if (sen_db_lock(s, -1)) {
@@ -862,11 +929,12 @@ sen_db_store_del_trigger(sen_db_store *e, sen_db_store_rel_spec *t)
   newspec_len = SEN_DB_STORE_SPEC_SIZE(n_triggers);
   if (!(newspec = SEN_GMALLOC(newspec_len))) {
     sen_db_unlock(s);
+    sen_ja_unref(s->values, e->id, spec, spec_len);
     return sen_memory_exhausted;
   }
   memcpy(newspec, spec, newspec_len);
   newspec->n_triggers = n_triggers;
-  sen_ja_unref(s->values, e->id);
+  sen_ja_unref(s->values, e->id, spec, spec_len);
   for (t = newspec->triggers, r = e->triggers; r; t++, r = r->next) {
     t->type = r->type;
     t->target = r->target;
@@ -958,7 +1026,7 @@ sen_db_store_remove(sen_db *s, const char *name)
   sen_db_store *e;
   uint32_t spec_len;
   char buffer[PATH_MAX];
-  const sen_db_store_spec *spec;
+  sen_db_store_spec *spec;
   if (!s || !(id = sen_sym_at(s->keys, name))) { return sen_invalid_argument; }
   if ((e = sen_array_at(&s->stores, id)) && (e->type)) { sen_db_store_close(e); }
   if (!(spec = sen_ja_ref(s->values, id, &spec_len))) {
@@ -980,6 +1048,7 @@ sen_db_store_remove(sen_db *s, const char *name)
   }
   if (sen_db_lock(s, -1)) {
     SEN_LOG(sen_log_crit, "sen_db_store_remove: lock failed");
+    sen_ja_unref(s->values, id, spec, spec_len);
     return sen_abnormal_error;
   }
   MUTEX_LOCK(s->lock);
@@ -1009,7 +1078,7 @@ sen_db_store_remove(sen_db *s, const char *name)
   default :
     break;
   }
-  sen_ja_unref(s->values, id);
+  sen_ja_unref(s->values, id, spec, spec_len);
   {
     sen_ja_einfo einfo;
     memset(&einfo, 0, sizeof(sen_ja_einfo));
