@@ -77,14 +77,13 @@ sen_ja *sen_ja_open(const char *path);
 sen_rc sen_ja_info(sen_ja *ja, unsigned int *max_element_size);
 sen_rc sen_ja_close(sen_ja *ja);
 sen_rc sen_ja_remove(const char *path);
-sen_rc sen_ja_put(sen_ja *ja, sen_id id, const void *value, int value_len, int flags);
+sen_rc sen_ja_put(sen_ja *ja, sen_id id, void *value, int value_len, int flags);
 int sen_ja_at(sen_ja *ja, sen_id id, void *valbuf, int buf_size);
 
-const void *sen_ja_ref(sen_ja *ja, sen_id id, uint32_t *value_len);
-sen_rc sen_ja_unref(sen_ja *ja, sen_id id);
+void *sen_ja_ref(sen_ja *ja, sen_id id, uint32_t *value_len);
+sen_rc sen_ja_unref(sen_ja *ja, sen_id id, void *value, uint32_t value_len);
 
 int sen_ja_size(sen_ja *ja, sen_id id);
-sen_rc sen_ja_alloc(sen_ja *ja, int element_size, sen_ja_einfo *einfo, void **value);
 sen_rc sen_ja_replace(sen_ja *ja, sen_id id, sen_ja_einfo *ei);
 
 /**** db ****/
