@@ -1894,3 +1894,22 @@ sen_lbuf_fin(sen_lbuf *buf)
   }
   return sen_success;
 }
+
+sen_rc
+sen_substring(char **str, char **str_end, int start, int end, sen_encoding encoding)
+{
+  int i;
+  size_t l;
+  char *s = *str, *e = *str_end;
+  for (i = 0; s < e; i++, s += l) {
+    if (i == start) { *str = s; }
+    if (!(l = sen_str_charlen_nonnull(s, e, encoding))) {
+      return sen_invalid_argument;
+    }
+    if (i == end) {
+      *str_end = s;
+      break;
+    }
+  }
+  return sen_success;
+}
