@@ -40,14 +40,6 @@ enum {
   sen_str_others
 };
 
-typedef struct {
-  uint32_t ctype;
-  char *start;
-  uint32_t length;
-} sen_substr;
-
-typedef struct _sen_str_splitter sen_str_splitter;
-
 typedef struct {
   const char *orig;
   size_t orig_blen;
@@ -76,12 +68,6 @@ typedef struct {
   sen_str_getopt_op op;
 } sen_str_getopt_opt;
 
-sen_str_splitter *sen_str_splitter_open(char *str);
-char *sen_str_splitter_next(sen_str_splitter *s, int *ctype);
-sen_rc sen_str_splitter_close(sen_str_splitter *s);
-
-int sen_str_split(char *str, sen_substr *res, int max, char **rest);
-
 int sen_str_get_prefix_order(const char *str);
 
 sen_nstr *sen_nstr_open(const char *str, size_t str_len, sen_encoding encoding, int flags);
@@ -157,6 +143,8 @@ sen_rc sen_lbuf_fin(sen_lbuf *buf);
 char *sen_str_itob(sen_id id, char *p);
 sen_id sen_str_btoi(char *b);
 
+sen_rc sen_substring(char **str, char **str_end, int start, int end, sen_encoding encoding);
+
 void sen_logger_fin(void);
 
 #ifdef __cplusplus
