@@ -14,8 +14,8 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-#ifndef _SENNA_H
-#define _SENNA_H
+#ifndef SENNA_H
+#define SENNA_H
 
 #ifdef  __cplusplus
 extern "C" {
@@ -32,6 +32,7 @@ extern "C" {
 #define SEN_INDEX_ENABLE_SUFFIX_SEARCH          0x0100
 #define SEN_INDEX_DISABLE_SUFFIX_SEARCH         0x0200
 #define SEN_INDEX_WITH_VGRAM                    0x1000
+#define SEN_INDEX_SHARED_LEXICON                0x2000
 #define SEN_INDEX_WITH_VACUUM                   0x8000
 
 /* 16 tokenizers can be registered */
@@ -42,6 +43,8 @@ extern "C" {
 #define SEN_SYM_WITH_SIS                        0x80000000
 
 #define SEN_SNIP_NORMALIZE                      0x0001
+#define SEN_SNIP_COPY_TAG                       0x0002
+#define SEN_QUERY_SCAN_NORMALIZE                SEN_SNIP_NORMALIZE
 
 #define SEN_LEX_NGRAM_UNIT_SIZE                 2
 
@@ -79,6 +82,9 @@ extern "C" {
 #ifndef SEN_QUERY_QUOTER
 #define SEN_QUERY_QUOTER '"'
 #endif /* SEN_QUERY_QUOTER */
+#ifndef SEN_QUERY_ESCAPE
+#define SEN_QUERY_ESCAPE '\\'
+#endif /* SEN_QUERY_ESCAPE */
 
 #define SEN_SYM_NIL 0
 
@@ -88,7 +94,10 @@ typedef enum {
   sen_invalid_format,
   sen_file_operation_error,
   sen_invalid_argument,
-  sen_other_error
+  sen_other_error,
+  sen_external_error,
+  sen_internal_error,
+  sen_abnormal_error
 } sen_rc;
 
 typedef enum {
@@ -145,6 +154,9 @@ typedef enum {
   sen_log_dump
 } sen_log_level;
 
+typedef struct _sen_db sen_db;
+typedef struct _sen_ctx sen_ctx;
+typedef struct _sen_ctx_info sen_ctx_info;
 typedef struct _sen_set sen_set;
 typedef struct _sen_sym sen_sym;
 typedef struct _sen_inv sen_inv;
@@ -171,6 +183,14 @@ typedef unsigned sen_id;
 
 typedef sen_set_eh sen_recordh;
 
+struct _sen_ctx_info {
+  int fd;
+  unsigned int com_status;
+  unsigned int com_info;
+  struct _sen_rbuf *outbuf;
+  unsigned char stat;
+};
+
 struct _sen_index {
   int foreign_flags;
   sen_sym *keys;
@@ -193,6 +213,7 @@ struct _sen_records {
   sen_recordh *sorted;
   int ignore_deleted_records;
   void *userdata;
+  sen_id subrec_id;
 };
 
 struct _sen_value {
@@ -218,7 +239,7 @@ struct _sen_select_optarg {
 
 struct _sen_group_optarg {
   sen_sort_mode mode;
-  int (*func)(sen_records *, const sen_recordh *, void **, void *);
+  int (*func)(sen_records *, const sen_recordh *, void *, void *);
   void *func_arg;
   int key_size;
 };
@@ -254,6 +275,30 @@ struct _sen_logger_info {
 
 typedef int (*query_term_callback)(const char *, unsigned int, void *);
 
+
+/******** query language API ********/
+
+sen_db *sen_db_create(const char *path, int flags, sen_encoding encoding);
+sen_db *sen_db_open(const char *path);
+sen_rc sen_db_close(sen_db *s);
+
+#define SEN_CTX_MORE  0x01
+#define SEN_CTX_TAIL  0x02
+#define SEN_CTX_HEAD  0x04
+#define SEN_CTX_QUIET 0x08
+#define SEN_CTX_QUIT  0x10
+
+#define SEN_CTX_USEQL 1
+#define SEN_CTX_BATCHMODE 2
+
+sen_ctx *sen_ctx_open(sen_db *db, int flags);
+sen_ctx *sen_ctx_connect(const char *host, int port, int flags);
+sen_rc sen_ctx_load(sen_ctx *c, const char *path);
+sen_rc sen_ctx_send(sen_ctx *c, char *str, unsigned int str_len, int flags);
+sen_rc sen_ctx_recv(sen_ctx *c, char **str, unsigned int *str_len, int *flags);
+sen_rc sen_ctx_close(sen_ctx *c);
+sen_rc sen_ctx_info_get(sen_ctx *c, sen_ctx_info *info);
+
 /******** basic API ********/
 
 sen_rc sen_init(void);
@@ -328,7 +373,8 @@ sen_rc sen_index_info(sen_index *i, int *key_size, int *flags,
                       int *initial_n_segments, sen_encoding *encoding,
                       unsigned *nrecords_keys, unsigned *file_size_keys,
                       unsigned *nrecords_lexicon, unsigned *file_size_lexicon,
-                      unsigned *inv_seg_size, unsigned *inv_chunk_size);
+                      unsigned long long *inv_seg_size,
+                      unsigned long long *inv_chunk_size);
 int sen_index_path(sen_index *i, char *pathbuf, int buf_size);
 /*
 sen_set *sen_index_related_terms(sen_index *index, const char *string,
@@ -343,6 +389,14 @@ unsigned int sen_query_rest(sen_query *q, const char ** const rest);
 sen_rc sen_query_close(sen_query *q);
 sen_rc sen_query_exec(sen_index *i, sen_query *q, sen_records *r, sen_sel_operator op);
 void sen_query_term(sen_query *q, query_term_callback func, void *func_arg);
+sen_rc sen_query_scan(sen_query *q, const char **strs, unsigned int *str_lens,
+                      unsigned int nstrs, int flags, int *found, int *score);
+sen_snip *sen_query_snip(sen_query *query, int flags,
+                         unsigned int width, unsigned int max_results,
+                         unsigned int n_tags,
+                         const char **opentags, unsigned int *opentag_lens,
+                         const char **closetags, unsigned int *closetag_lens,
+                         sen_snip_mapping *mapping);
 
 sen_rc sen_index_del(sen_index *i, const void *key);
 
@@ -445,21 +499,8 @@ if (sen_logger_pass(SEN_LOG_DEFAULT_LEVEL)) {\
   sen_logger_put(SEN_LOG_DEFAULT_LEVEL, __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__);\
 }
 
-/* sen_com_client */
-
-typedef struct _sen_com_sqtp sen_com_client;
-
-sen_rc sen_com_init(void);
-void sen_com_fin(void);
-sen_com_client *sen_com_client_open(const char *host, int port);
-sen_rc sen_com_client_send(sen_com_client *c, const char *str, unsigned int str_len,
-                           int morep);
-sen_rc sen_com_client_recv(sen_com_client *c, char **str, unsigned int *str_len,
-                           int *morep);
-sen_rc sen_com_client_close(sen_com_client *c);
-
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* _SENNA_H */
+#endif /* SENNA_H */
