@@ -24,12 +24,11 @@
 #endif /* HAVE_SYS_WAIT_H */
 
 #define DEFAULT_PORT 10041
-#define DEFAULT_PATH "." PATH_SEPARATOR "senna.db"
 #define DEFAULT_DEST "localhost"
 
-int port = DEFAULT_PORT;
-int batchmode;
-sen_encoding enc = sen_enc_default;
+static int port = DEFAULT_PORT;
+static int batchmode;
+static sen_encoding enc = sen_enc_default;
 
 static void
 usage(void)
@@ -40,11 +39,12 @@ usage(void)
           "  -a:                 run in standalone mode (default)\n"
           "  -c:                 run in client mode\n"
           "  -s:                 run in server mode\n"
+          "  -e:                 encoding for new database [none|euc|utf8|sjis|latin1|koi8r]\n"
           "  -p <port number>:   server port number (default: %d)\n"
           "dest: <db pathname> or <dest hostname>\n"
-          "  <db pathname>: when standalone/server mode (default: \"%s\")\n"
+          "  <db pathname>: when standalone/server mode\n"
           "  <dest hostname>: when client mode (default: \"%s\")\n",
-          DEFAULT_PORT, DEFAULT_PATH, DEFAULT_DEST);
+          DEFAULT_PORT, DEFAULT_DEST);
 }
 
 inline static void
@@ -53,69 +53,30 @@ prompt(void)
   if (!batchmode) { fputs("> ", stderr); }
 }
 
-static void
-output(sen_ctx *c, int flags, void *dummy)
-{
-  sen_com_sqtp_header header;
-  sen_rbuf *buf = &c->outbuf;
-  header.size = SEN_RBUF_VSIZE(buf);
-  header.flags = (flags & SEN_QL_MORE) ? SEN_COM_SQTP_MORE : SEN_COM_SQTP_TAIL;
-  sen_com_sqtp_send((sen_com_sqtp *)c->data.ptr, &header, buf->head);
-  SEN_RBUF_REWIND(buf);
-}
-
-static void
-msg_handler(sen_com_event *ev, sen_com *c)
-{
-  sen_com_sqtp *cs = (sen_com_sqtp *)c;
-  sen_ctx *ctx = (sen_ctx *)cs->userdata;
-  if (cs->rc) { goto exit; }
-  if (!ctx) {
-    ctx = sen_ctx_open((sen_db *)ev->userdata, SEN_CTX_USEQL);
-    if (!ctx) { goto exit; }
-    sen_ctx_recv_handler_set(ctx, output, cs);
-    sen_ctx_load(ctx, NULL);
-    cs->userdata = ctx;
-  }
-  {
-    char *body = SEN_COM_SQTP_MSG_BODY(&cs->msg);
-    sen_com_sqtp_header *header = SEN_COM_SQTP_MSG_HEADER(&cs->msg);
-    uint32_t size = header->size;
-    uint16_t flags = header->flags;
-    sen_ctx_send(ctx, body, size, flags);
-    return;
-  }
-exit :
-  SEN_LOG(sen_log_notice, "connection closed..");
-  if (ctx) { sen_ctx_close(ctx); }
-  sen_com_sqtp_close(ev, cs);
-}
-
-#define BUFSIZE 0x100000
-#define MAX_CON 0x100
+#define BUFSIZE 0x1000000
 
 static int
 do_alone(char *path)
 {
   int rc = -1;
-  sen_db *db;
-  if ((db = sen_db_open(path ? path : DEFAULT_PATH)) ||
-      (path && (db = sen_db_create(path, 0, enc)))) {
+  sen_db *db = NULL;
+  if (!path || (db = sen_db_open(path)) || (db = sen_db_create(path, 0, enc))) {
     sen_ctx *ctx;
     if ((ctx = sen_ctx_open(db, SEN_CTX_USEQL|(batchmode ? SEN_CTX_BATCHMODE : 0)))) {
-      sen_ctx_info info;
-      sen_ctx_info_get(ctx, &info);
-      if (!sen_rbuf_reinit(info.outbuf, BUFSIZE)) {
+      char *buf = SEN_MALLOC(BUFSIZE);
+      if (buf) {
         sen_ctx_recv_handler_set(ctx, sen_ctx_stream_out_func, stdout);
         sen_ctx_load(ctx, NULL);
-        while (prompt(), fgets(info.outbuf->head, SEN_RBUF_WSIZE(info.outbuf), stdin)) {
-          uint32_t size = strlen(info.outbuf->head) - 1;
-          info.outbuf->head[size] = '\0';
-          sen_ctx_send(ctx, info.outbuf->head, size, 0);
+        while (prompt(), fgets(buf, BUFSIZE, stdin)) {
+          uint32_t size = strlen(buf) - 1;
+          buf[size] = '\0';
+          sen_ctx_send(ctx, buf, size, 0);
+          if (ctx->stat == SEN_CTX_QUIT) { break; }
         }
+        SEN_FREE(buf);
         rc = 0;
       } else {
-        fprintf(stderr, "sen_rbuf_reinit failed (%d)\n", BUFSIZE);
+        fprintf(stderr, "sen_malloc failed (%d)\n", BUFSIZE);
       }
       sen_ctx_close(ctx);
     } else {
@@ -128,6 +89,18 @@ do_alone(char *path)
   return rc;
 }
 
+#define BATCHMODE(ctx) do {\
+  int flags;\
+  unsigned int str_len;\
+  char *str, *query = "(batchmode #t)";\
+  sen_ctx_send(ctx, query, strlen(query), 0);\
+  do {\
+    if (sen_ctx_recv(ctx, &str, &str_len, &flags)) {\
+      fprintf(stderr, "sen_ctx_recv failed\n");\
+    }\
+  } while ((flags & SEN_CTX_MORE));\
+} while (0)
+
 static int
 do_client(char *hostname)
 {
@@ -137,6 +110,7 @@ do_client(char *hostname)
     sen_ctx_info info;
     sen_ctx_info_get(ctx, &info);
     if (!sen_rbuf_reinit(info.outbuf, BUFSIZE)) {
+      if (batchmode) { BATCHMODE(ctx); }
       while (prompt(), fgets(info.outbuf->head, SEN_RBUF_WSIZE(info.outbuf), stdin)) {
         int flags;
         char *str;
@@ -151,8 +125,10 @@ do_client(char *hostname)
           if (str_len) {
             fwrite(str, 1, str_len, stdout);
             putchar('\n');
+            fflush(stdout);
           }
         } while ((flags & SEN_CTX_MORE));
+        if (ctx->stat == SEN_CTX_QUIT) { break; }
       }
       rc = 0;
     } else {
@@ -166,16 +142,174 @@ exit :
   return rc;
 }
 
-// todo : use thread
+/* server */
+
+static void
+output(sen_ctx *ctx, int flags, void *arg)
+{
+  static uint32_t info = 0;
+  sen_com_sqtp *cs = arg;
+  sen_com_sqtp_header header;
+  sen_rbuf *buf = &ctx->outbuf;
+  header.size = SEN_RBUF_VSIZE(buf);
+  header.flags = (flags & SEN_CTX_MORE) ? SEN_CTX_MORE : SEN_CTX_TAIL;
+  header.qtype = 1;
+  header.level = 0;
+  header.status = 0;
+  header.info = info++; /* for debug */
+  if (ctx->stat == SEN_CTX_QUIT) { cs->com.status = sen_com_closing; }
+  sen_com_sqtp_send(cs, &header, buf->head);
+  SEN_RBUF_REWIND(buf);
+}
+
+static void
+errout(sen_com_sqtp *cs, char *msg)
+{
+  sen_com_sqtp_header header;
+  header.size = strlen(msg);
+  header.flags = SEN_CTX_TAIL;
+  sen_com_sqtp_send(cs, &header, msg);
+  SEN_LOG(sen_log_error, "errout: %s", msg);
+}
+
+inline static void
+do_msg(sen_com_event *ev, sen_com_sqtp *cs)
+{
+  sen_ctx *ctx = (sen_ctx *)cs->userdata;
+  if (!ctx) {
+    ctx = sen_ctx_open((sen_db *)ev->userdata, SEN_CTX_USEQL);
+    if (!ctx) {
+      cs->com.status = sen_com_closing;
+      errout(cs, "*** ERROR: ctx open failed");
+      return;
+    }
+    sen_ctx_recv_handler_set(ctx, output, cs);
+    sen_ctx_load(ctx, NULL);
+    cs->userdata = ctx;
+  }
+  {
+    char *body = SEN_COM_SQTP_MSG_BODY(&cs->msg);
+    sen_com_sqtp_header *header = SEN_COM_SQTP_MSG_HEADER(&cs->msg);
+    uint32_t size = header->size;
+    uint16_t flags = header->flags;
+    sen_ctx_send(ctx, body, size, flags);
+    if (ctx->stat == SEN_CTX_QUIT) {
+      cs->com.status = sen_com_closing;
+      sen_ctx_close(ctx);
+      cs->userdata = NULL;
+    } else {
+      cs->com.status = sen_com_idle;
+    }
+    return;
+  }
+}
+
+/* query queue */
+
+static sen_mutex q_mutex;
+static sen_cond q_cond;
+
+typedef struct {
+  uint8_t head;
+  uint8_t tail;
+  sen_com_sqtp *v[0x100];
+} queue;
+
+static void
+queue_init(queue *q)
+{
+  q->head = 0;
+  q->tail = 0;
+}
+
+static sen_rc
+queue_enque(queue *q, sen_com_sqtp *c)
+{
+  uint8_t i = q->tail + 1;
+  if (i == q->head) { return sen_other_error; }
+  q->v[q->tail] = c;
+  q->tail = i;
+  return sen_success;
+}
+
+static queue qq;
+
+/* worker thread */
+
+#define MAX_NFTHREADS 0x04
+
+static uint32_t nthreads = 0, nfthreads = 0;
+
+static void * CALLBACK
+thread_start(void *arg)
+{
+  sen_com_sqtp *cs;
+  SEN_LOG(sen_log_notice, "thread start (%d/%d)", nfthreads, nthreads + 1);
+  MUTEX_LOCK(q_mutex);
+  nthreads++;
+  do {
+    nfthreads++;
+    while (qq.tail == qq.head) { COND_WAIT(q_cond, q_mutex); }
+    nfthreads--;
+    cs = qq.v[qq.head++];
+    MUTEX_UNLOCK(q_mutex);
+    if (cs) { do_msg((sen_com_event *) arg, cs); }
+    MUTEX_LOCK(q_mutex);
+  } while (nfthreads < MAX_NFTHREADS);
+  nthreads--;
+  MUTEX_UNLOCK(q_mutex);
+  SEN_LOG(sen_log_notice, "thread end (%d/%d)", nfthreads, nthreads);
+  return NULL;
+}
+
+static void
+msg_handler(sen_com_event *ev, sen_com *c)
+{
+  sen_com_sqtp *cs = (sen_com_sqtp *)c;
+  if (cs->rc) {
+    sen_ctx *ctx = (sen_ctx *)cs->userdata;
+    SEN_LOG(sen_log_notice, "connection closed..");
+    if (ctx) { sen_ctx_close(ctx); }
+    sen_com_sqtp_close(ev, cs);
+    return;
+  }
+  {
+    int i = 0;
+    while (queue_enque(&qq, (sen_com_sqtp *)c)) {
+      if (i) {
+        SEN_LOG(sen_log_notice, "queue is full try=%d qq(%d-%d) thd(%d/%d) %d", i, qq.head, qq.tail, nfthreads, nthreads, ev->set->n_entries);
+      }
+      if (++i == 100) {
+        errout((sen_com_sqtp *)c, "*** ERROR: query queue is full");
+        return;
+      }
+      usleep(1000);
+    }
+  }
+  MUTEX_LOCK(q_mutex);
+  if (!nfthreads && nthreads < MAX_NFTHREADS) {
+    sen_thread thread;
+    MUTEX_UNLOCK(q_mutex);
+    if (THREAD_CREATE(thread, thread_start, ev)) { GSERR("pthread_create"); }
+    return;
+  }
+  COND_SIGNAL(q_cond);
+  MUTEX_UNLOCK(q_mutex);
+}
+
+#define MAX_CON 0x10000
+
 static int
 server(char *path)
 {
   int rc = -1;
   sen_com_event ev;
+  MUTEX_INIT(q_mutex);
+  COND_INIT(q_cond);
+  queue_init(&qq);
   if (!sen_com_event_init(&ev, MAX_CON, sizeof(sen_com_sqtp))) {
-    sen_db *db;
-    if ((db = sen_db_open(path ? path : DEFAULT_PATH)) ||
-        (path && (db = sen_db_create(path, 0, enc)))) {
+    sen_db *db = NULL;
+    if (!path || (db = sen_db_open(path)) || (db = sen_db_create(path, 0, enc))) {
       sen_com_sqtp *cs;
       ev.userdata = db;
       if ((cs = sen_com_sqtp_sopen(&ev, port, msg_handler))) {
@@ -196,10 +330,10 @@ server(char *path)
   return rc;
 }
 
-#ifndef WIN32
 static int
 do_server(char *path)
 {
+#ifndef WIN32
   pid_t pid;
   switch (fork()) {
   case 0:
@@ -221,9 +355,9 @@ do_server(char *path)
     fprintf(stderr, "%d\n", pid);
     _exit(0);
   }
+#endif /* WIN32 */
   return server(path);
 }
-#endif /* WIN32 */
 
 enum {
   mode_alone,
@@ -232,10 +366,19 @@ enum {
   mode_usage
 };
 
+#define SET_LOGLEVEL(x) do {\
+  static sen_logger_info info;\
+  info.max_level = (x);\
+  info.flags = SEN_LOG_TIME|SEN_LOG_MESSAGE;\
+  info.func = NULL;\
+  info.func_arg = NULL;\
+  sen_logger_info_set(&info);\
+} while(0)
+
 int
 main(int argc, char **argv)
 {
-  char *portstr = NULL, *encstr = NULL;
+  char *portstr = NULL, *encstr = NULL, *loglevel = NULL;
   int r, i, mode = mode_alone;
   static sen_str_getopt_opt opts[] = {
     {'p', NULL, NULL, 0, getopt_op_none},
@@ -244,10 +387,12 @@ main(int argc, char **argv)
     {'a', NULL, NULL, mode_alone, getopt_op_update},
     {'c', NULL, NULL, mode_client, getopt_op_update},
     {'s', NULL, NULL, mode_server, getopt_op_update},
+    {'l', NULL, NULL, 0, getopt_op_none},
     {'\0', NULL, NULL, 0, 0}
   };
   opts[0].arg = &portstr;
   opts[1].arg = &encstr;
+  opts[6].arg = &loglevel;
   i = sen_str_getopt(argc, argv, opts, &mode);
   if (i < 0) { mode = mode_usage; }
   if (portstr) { port = atoi(portstr); }
@@ -281,6 +426,8 @@ main(int argc, char **argv)
   }
   batchmode = !isatty(0);
   if (sen_init()) { return -1; }
+  sen_gctx.encoding = enc; /* todo : make it api */
+  if (loglevel) { SET_LOGLEVEL(atoi(loglevel)); }
   switch (mode) {
   case mode_alone :
     r = do_alone(argc <= i ? NULL : argv[i]);
@@ -289,11 +436,7 @@ main(int argc, char **argv)
     r = do_client(argc <= i ? DEFAULT_DEST : argv[i]);
     break;
   case mode_server :
-#ifdef WIN32
-    r = server(argc <= i ? NULL : argv[i]);
-#else /* WIN32 */
     r = do_server(argc <= i ? NULL : argv[i]);
-#endif /* WIN32 */
     break;
   default :
     usage(); r = -1;
