@@ -200,7 +200,8 @@ show_index_info(sen_index *i, FILE *fp)
 {
   int keysz, flags, inseg;
   sen_encoding enc;
-  unsigned int nkey, szkey, nlex, szlex, szseg, szchunk;
+  unsigned int nkey, szkey, nlex, szlex;
+  unsigned long long szseg, szchunk;
   if (sen_index_info(i, &keysz, &flags, &inseg, &enc,
                      &nkey, &szkey, &nlex, &szlex, &szseg, &szchunk)) {
     fprintf(stderr, "sen_index_info failed...\n");
@@ -210,17 +211,17 @@ show_index_info(sen_index *i, FILE *fp)
   /* TODO: need sen_enc2str */
   fprintf(fp,
           "  Index file infomation:\n"
-          "   - lock                    : %10u\n"
-          "   - key_size                : %10d\n"
-          "   - flags                   : %10d\n"
-          "   - initial_n_segments      : %10d\n"
-          "   - encoding                : %10s\n"
-          "   - nrecords_keys           : %10u\n"
-          "   - file_size_keys          : %10u\n"
-          "   - nrecords_lexicon        : %10u\n"
-          "   - file_size_lexicon       : %10u\n"
-          "   - inv_segment_size        : %10u\n"
-          "   - inv_chunk_size          : %10u\n",
+          "   - lock                    : %12u\n"
+          "   - key_size                : %12d\n"
+          "   - flags                   : %12d\n"
+          "   - initial_n_segments      : %12d\n"
+          "   - encoding                : %12s\n"
+          "   - nrecords_keys           : %12u\n"
+          "   - file_size_keys          : %12u\n"
+          "   - nrecords_lexicon        : %12u\n"
+          "   - file_size_lexicon       : %12u\n"
+          "   - inv_segment_size        : %12llu\n"
+          "   - inv_chunk_size          : %12llu\n",
           *i->keys->lock, keysz, flags, inseg,
           sen_enctostr(enc), nkey, szkey,
           nlex, szlex, szseg, szchunk);
@@ -543,7 +544,7 @@ senchk_filelist_get(senchk_filelist *fl, const char *path)
       while ((cid = sen_sym_next(db->keys, cid)) != SEN_SYM_NIL) {
         uint32_t spec_len;
         char buffer[PATH_MAX];
-        const sen_db_store_spec *spec;
+        sen_db_store_spec *spec;
         if (!(spec = sen_ja_ref(db->values, cid, &spec_len))) {
           /* TODO: implement */
           return 0;
@@ -599,6 +600,7 @@ senchk_filelist_get(senchk_filelist *fl, const char *path)
         default:
           break;
         }
+        sen_ja_unref(db->values, cid, spec, spec_len);
       }
       sen_db_close(db);
     }
